#!/bin/bash -u

#
#	WARNING: Do Not Remove This Section
#
#	$RCSfile: statistics.sh,v $
#	$Revision: 1.3 $
#	$Date: 2006/10/16 12:43:32 $
#	$Author: zapreevis $
#	
#	MRMC is a model checker for discrete-time and continuous-time Markov reward models.
#	It supports reward extensions of PCTL and CSL (PRCTL and CSRL), and allows for the
#	automated verification of properties concerning long-run and instantaneous rewards
#	as well as cumulative rewards.
#	
#	Copyright (C) The University of Twente, 2004-2006.
#	Authors: Ivan Zapreev
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#	Our contacts:
#		Formal Methods and Tools Group, University of Twente,
#		P.O. Box 217, 7500 AE Enschede, The Netherlands,
#		Phone: +31 53 4893767, Fax: +31 53 4893247,
#		E-mail: mrmc@cs.utwente.nl
#
#	Source description: This script runs tests
#

TESTS=`eval cat ${1} | sed -e '/#.*/d'`

INPUTS=`eval cat ${2} | sed -e '/#.*/d'`

##########################################
#%%%%%%%%%%%%EXECUTE TESTS%%%%%%%%%%%%%%%#
##########################################

export PATH=.:${PATH}		#Add local directory to the path, otherwise the tests will not run

#remove previous statistics
rm -f *.statistics

#Perform data collection and processing for the statistics.
echo "------------ Collecting statistics and preparing data ------------"
for test in ${TESTS}
do
	cd ${test}
	TEST_NAME=${test}
	echo "${TEST_NAME}: "
	for input in ${INPUTS}
	do
		echo "  ${input}:"
		options_line=1	#Initialize the line counter
		cat ../option_list | sed -e '/#.*/d' | while read options; do
			#Used for constructing the test case name
			OPTS=`eval echo ${options} | sed -e 's/ /./'`
			TEST_CASE_NAME=${TEST_NAME}.${OPTS}.${input}
			
			echo -n "    ${TEST_CASE_NAME}"
			
			#Read multiples corresponding to the options list
			multiple=`eval cat ../multiple_list | sed -e '/#.*/d' | sed -e "${options_line}!d"`
			let "options_line+=1"	#Increment the line counter
			
			#Process the resultes files: just live the consumed time
			#and compute averages appending them to the previous resultes files
			cat ${TEST_CASE_NAME}.results | sed -f ${PERFORMANCE_SED_SCRIPTS}/sed.rules | awk -v mult=${multiple} -v out_file="../${input}.${OPTS}.statistics" -f ${PERFORMANCE_AWK_SCRIPTS}/average.awk
			
			echo " ..........DONE"
		done
	done
	cd ..
done

echo "Converting the statistics into the gnuplot data files:"
for input in ${INPUTS}
do
	echo "  ${input}.dat: "
	
	#Remove the old file
	rm -f ${input}.dat;
	
	#Prepare the input parameters for the awk script
	OUT=${input}.dat;
	PARAM=${TEST_SUBSUITE_NAME}.param;
	
	#Place the input files into the arrange_table.awk input
	#in the same order as we have options in the optins_list
	#such that the columns are ordered by the MRMC options
	rm -f stat.files.tmp
	cat option_list | sed -e '/#.*/d' | while read options; do
		OPTS=`eval echo ${options} | sed -e 's/ /./'`
		echo -n ${input}.${OPTS}.statistics" " >> stat.files.tmp
	done
	STAT=`eval cat stat.files.tmp`
	rm -f stat.files.tmp

	#Arrange the dat file with all gnuplot data
	echo "${PARAM} ${STAT}" | awk -v out_file_name=${OUT} -v microseconds=${MICROSECONDS} -f ${PERFORMANCE_AWK_SCRIPTS}/arrange_table.awk
	
	#Get rid of "," which whould be substituted by "."
	#Because awk and gnuplot for numbers is a little different
	cat ${OUT} | sed -e 's/,/./g' > ${OUT}.tmp
	rm -f ${OUT}
	mv ${OUT}.tmp ${OUT}

	#Compute min max values for the range
	MIN_MAX=`eval echo ${STAT} | awk -v microseconds=${MICROSECONDS} -f ${PERFORMANCE_AWK_SCRIPTS}/min_max.awk`
	MIN=`eval echo \"${MIN_MAX}\" | sed -e 's/;.*//;s/,/./'`
	MAX=`eval echo \"${MIN_MAX}\" | sed -e 's/.*;//;s/,/./'`
	
	#Modifu the ${input}.gnuplot file, i.e. substitute MIN, MAX for required values
	#and also set the proper name for the INPUT file
	if test ${MICROSECONDS} -eq "1"; then
		TIME_UNIT="micro sec(s)"
	elif test ${MICROSECONDS} -eq "1000"; then
		TIME_UNIT="milli sec(s)"
	elif test ${MICROSECONDS} -eq "1000000"; then
		TIME_UNIT="seconds"
	elif test ${MICROSECONDS} -eq "60000000"; then
		TIME_UNIT="minutes"
	elif test ${MICROSECONDS} -eq "3600000000"; then
		TIME_UNIT="hours"
	else
		TIME_UNIT="1 time unit = ${MICROSECONDS} micro sec(s)"
	fi
	cat ${input}.gnuplot | sed -e "s/MIN/${MIN}/;s/MAX/${MAX}/;s/INPUT/${input}/;s/TIME_UNIT/${TIME_UNIT}/" > ${input}.gnuplot.tmp
	
	#Generate the eps
	echo -n "  Generating: ${input}.eps"
	gnuplot ${input}.gnuplot.tmp
	echo " ..........DONE"
	
	rm -f ${input}.gnuplot.tmp
done