#!/bin/bash

#
#	WARNING: Do Not Remove This Section
#
#	$RCSfile: test_all.sh,v $
#	$Revision: 1.11 $
#	$Date: 2006/10/16 12:36:40 $
#	$Author: zapreevis $
#	
#	MRMC is a model checker for discrete-time and continuous-time Markov reward models.
#	It supports reward extensions of PCTL and CSL (PRCTL and CSRL), and allows for the
#	automated verification of properties concerning long-run and instantaneous rewards
#	as well as cumulative rewards.
#	
#	Copyright (C) The University of Twente, 2004-2006.
#	Authors: Ivan Zapreev
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#	Our contacts:
#		Formal Methods and Tools Group, University of Twente,
#		P.O. Box 217, 7500 AE Enschede, The Netherlands,
#		Phone: +31 53 4893767, Fax: +31 53 4893247,
#		E-mail: mrmc@cs.utwente.nl
#
#	Source description: Runs tests.
#

#####################################
#### Set the following variables ####
#####################################

source settings.cfg

#####################################

################
##### MAIN #####
################
HELP_STR="help"
VALGRIND_STR="valgrind"
ALL_STR="all"
INTERNAL_STR="internal"
FUNCTIONAL_STR="functional"
PERFORMANCE_STR="performance"

help ()
{
	echo "Usage: ${0}  [options]"
	echo "Options:"
	echo "  -${ALL_STR}: runs all tests"
	echo "  -${INTERNAL_STR}: runs internal tests"
	echo "  -${FUNCTIONAL_STR}: runs functional tests"
	echo "  -${PERFORMANCE_STR}: runs performance tests"
	echo "  -${VALGRIND_STR}: turnes on the Valgrind, should be used with mrmc compiled with '-O0 -ggdb -g' options"
}

#Valgrind is a memory profiler
valgrind ()
{
	echo "========= VaLGRIND IS ON ========="
	export VALGRIND_LIB="${VALGRIND_HOME}/.in_place"
	export MRMC="${VALGRIND_HOME}/coregrind/valgrind --leak-check=yes ${MRMC}"	#This path is regarding being in the test case directory
}

internal ()
{
	echo "========= Running Internal Tests ========="
	cd internal_tests
	./test.sh test_list
	cd ..
}

functional () {
	echo "======== Running Functional Tests ========"
	cd functional_tests
	./test.sh test_list
	cd ..
}

performance ()
{
	echo "======== Running Performance Tests ========"
	echo "Each test will be repeated '${NUMBER_OF_PERFORMANCE_REPETITIONS}' times."
	cd performance_tests
	./test.sh test_list
	cd ..
}

all ()
{
	echo "========= ALL TEST MODE ========="
	internal;
	functional;
	performance;
}

HELP=""
VALGRIND=""
ALL=""
INTERNAL=""
FUNCTIONAL=""
PERFORMANCE=""

opt="$1"
while test ! -z "$opt" -a -z "${opt##-*}"; do
	opt=${opt#-}
	#echo Param: $opt
	case "$opt" in
		${HELP_STR}) HELP=1;;
		${VALGRIND_STR}) VALGRIND=1;;
		${ALL_STR}) ALL=1 ;;
		${INTERNAL_STR}) INTERNAL=1;;
		${FUNCTIONAL_STR}) FUNCTIONAL=1;;
		${PERFORMANCE_STR}) PERFORMANCE=1;;
	esac
	shift
	opt="$1"
done

if test ! -z "${VALGRIND}"; then
	valgrind;
fi

if test ! -z "${ALL}"; then
	all;
else
	if test ! -z "${INTERNAL}"; then
		internal;
	fi
	if test ! -z "${FUNCTIONAL}"; then
		functional;
	fi
	if test ! -z "${PERFORMANCE}"; then
		performance;
	fi
	if test ! -z "${HELP}" -o -z "${INTERNAL}" -a -z "${FUNCTIONAL}" -a -z "${PERFORMANCE}"; then
		help;
	fi
fi
